import { lazy } from 'react';
import { when } from 'mobx';
import DisplayType from '@jbrowse/core/pluggableElementTypes/DisplayType';
import ViewType from '@jbrowse/core/pluggableElementTypes/ViewType';
import Plugin from '@jbrowse/core/Plugin';
import { isAbstractMenuManager } from '@jbrowse/core/util';
import LineStyleIcon from '@mui/icons-material/LineStyle';
import { BaseLinearDisplay, BaseLinearDisplayComponent, baseLinearDisplayConfigSchema, } from './BaseLinearDisplay';
import { configSchemaFactory as linearBareDisplayConfigSchemaFactory, stateModelFactory as LinearBareDisplayStateModelFactory, } from './LinearBareDisplay';
import { stateModelFactory as linearGenomeViewStateModelFactory, renderToSvg, RefNameAutocomplete, SearchBox, ZoomControls, LinearGenomeView, } from './LinearGenomeView';
import { configSchema as linearBasicDisplayConfigSchemaFactory, modelFactory as linearBasicDisplayModelFactory, } from './LinearBasicDisplay';
import FeatureTrackF from './FeatureTrack';
import BasicTrackF from './BasicTrack';
export default class LinearGenomeViewPlugin extends Plugin {
    constructor() {
        super(...arguments);
        this.name = 'LinearGenomeViewPlugin';
        this.exports = {
            BaseLinearDisplayComponent,
            BaseLinearDisplay,
            baseLinearDisplayConfigSchema,
            SearchBox,
            ZoomControls,
            LinearGenomeView,
        };
    }
    install(pluginManager) {
        FeatureTrackF(pluginManager);
        BasicTrackF(pluginManager);
        pluginManager.addDisplayType(() => {
            const configSchema = linearBareDisplayConfigSchemaFactory(pluginManager);
            return new DisplayType({
                name: 'LinearBareDisplay',
                configSchema,
                stateModel: LinearBareDisplayStateModelFactory(configSchema),
                trackType: 'BasicTrack',
                viewType: 'LinearGenomeView',
                ReactComponent: BaseLinearDisplayComponent,
            });
        });
        pluginManager.addDisplayType(() => {
            const configSchema = linearBasicDisplayConfigSchemaFactory(pluginManager);
            return new DisplayType({
                name: 'LinearBasicDisplay',
                configSchema,
                stateModel: linearBasicDisplayModelFactory(configSchema),
                trackType: 'FeatureTrack',
                viewType: 'LinearGenomeView',
                ReactComponent: BaseLinearDisplayComponent,
            });
        });
        pluginManager.addViewType(() => new ViewType({
            name: 'LinearGenomeView',
            stateModel: linearGenomeViewStateModelFactory(pluginManager),
            ReactComponent: lazy(() => import('./LinearGenomeView/components/LinearGenomeView')),
        }));
        pluginManager.addToExtensionPoint('LaunchView-LinearGenomeView', 
        // @ts-ignore
        async ({ session, assembly, loc, tracks = [], }) => {
            try {
                const { assemblyManager } = session;
                const view = session.addView('LinearGenomeView', {});
                await when(() => !!view.volatileWidth);
                if (!assembly) {
                    throw new Error('No assembly provided when launching linear genome view');
                }
                const asm = await assemblyManager.waitForAssembly(assembly);
                if (!asm) {
                    throw new Error(`Assembly "${assembly}" not found when launching linear genome view`);
                }
                view.navToLocString(loc, assembly);
                const idsNotFound = [];
                tracks.forEach(track => {
                    try {
                        view.showTrack(track);
                    }
                    catch (e) {
                        if (`${e}`.match('Could not resolve identifier')) {
                            idsNotFound.push(track);
                        }
                        else {
                            throw e;
                        }
                    }
                });
                if (idsNotFound.length) {
                    throw new Error(`Could not resolve identifiers: ${idsNotFound.join(',')}`);
                }
            }
            catch (e) {
                session.notify(`${e}`, 'error');
                throw e;
            }
        });
    }
    configure(pluginManager) {
        if (isAbstractMenuManager(pluginManager.rootModel)) {
            pluginManager.rootModel.appendToSubMenu(['Add'], {
                label: 'Linear genome view',
                icon: LineStyleIcon,
                onClick: (session) => {
                    session.addView('LinearGenomeView', {});
                },
            });
        }
    }
}
export { baseLinearDisplayConfigSchema, linearBareDisplayConfigSchemaFactory, linearBasicDisplayConfigSchemaFactory, linearBasicDisplayModelFactory, renderToSvg, BaseLinearDisplayComponent, BaseLinearDisplay, RefNameAutocomplete, SearchBox, };
//# sourceMappingURL=index.js.map